////////////////////////////////////////
//
// Free for private and commercial use.
//
////////////////////////////////////////

#include <zneo.h>

#include "zneo2.h"
#include "sio.h"

int init_uart(int port,unsigned long freq, unsigned long baud)
{
    unsigned long brg;
    
	/* calculate the desired communication rate */
    brg = (freq+baud*8ul)/((unsigned long)baud * 16);
   
    switch( port )
    {
        case _UART0:
            /* 1. set the baud-rate generater */
            U0BR = brg;           
            /* 2. configure GPIO for alternate function */
            PAAFL |= 0x30;    // enable the UART0 Rx/Tx with the AF register
            /* 3. configure UART control register 1 */
            U0CTL1 = 0x00;    // clear for normal non-Multiprocessor operation
            /* 4. configure UART control register 1 */
            U0CTL0 = 0xc0;    // Transmit enable, Receive Enable, No Parity, 1 Stop
            break;
        case _UART1:
            /* 1. set the baud-rate generater */
            U1BR = brg;
            /* 2. configure GPIO for alternate function */
            PDAFL |= 0x30;    // enable the UART1 Rx/Tx with the AF register
            /* 3. configure UART control register 1 */
            U1CTL1 = 0x00;    // clear for normal non-Multiprocessor operation
            /* 4. configure UART control register 1 */
            U1CTL0 = 0xc0;    // Transmit enable, Receive Enable, No Parity, 1 Stop
            break;
        default:
            // Illegal port number. Will be caught by select_port
		    return 1;  // Return failure
            break;
    }

    return 0;  // Return success
}

static void send(char c)
{
    if ((PAAFL & 0x30) == 0x30)
	{
        /* UART0 */
        while (!(U0STAT0 & 0x04));  // Transmit Data register enabled
        U0TXD = c;                      // Send data
    }
    else if ((PDAFL & 0x30) == 0x30)
	{
        /* UART1 */
        while (!(U1STAT0 & 0x04));  // Transmit Data register enabled
        U1TXD = c;                      // Send data
    }
}

int putch(char c)
{
    if (c == '\n')
        send('\r');
    send(c);
    return 0;
}

int kbhit(void)
{
    if ((PAAFL & 0x30) == 0x30)
	{
        if (U0STAT0 & 0x80)           // check for receive data available
            return(1);
    }
    else if ((PDAFL & 0x30) == 0x30)
	{
        if (U1STAT0 & 0x80)           // check for receive data available
            return(1);
    }

    return 0;
}

char getch(void)
{
    reg_t ch;

    if ((PAAFL & 0x30) == 0x30)
	{
        while (!(U0STAT0 & 0x80));     // Wait for receive data available
        ch = U0RXD;
    }
    else if ((PDAFL & 0x30) == 0x30)
	{
        while (!(U1STAT0 & 0x80));     // Wait for receive data available
        ch = U1RXD;
    }
	else
		return 0;

    if (ch == '\r' || ch == ('\r'|0x80))
        return '\n';

    return ch;
}
